@@ -52,6 +52,7 @@ static void trans_proxy(int sock, struct sockaddr_in *addr,
 static int	get_site_from_headers(int sock, char *hostname, int hostlen,
 								  char *headers, int header_max,
 								  int *header_len);
+static void alarm_signal(int sig);
 
 /*
  * Command line switches.
@@ -186,6 +187,7 @@ int main(int argc, char **argv)
 			signal(SIGTSTP, SIG_IGN);
 			signal(SIGCONT, SIG_IGN);
 			signal(SIGPIPE, SIG_IGN);
+			signal(SIGALRM, alarm_signal);
 
 			/*
 			 * Drop back to an untrusted user.
@@ -233,6 +235,14 @@ int main(int argc, char **argv)
 		exit(1);
 	}
 
+	/*
+	 * Set the keepalive socket option to on.
+	 */
+	{
+		int		on = 1;
+		setsockopt(STDIN_FILENO, SOL_SOCKET, SO_KEEPALIVE, (char *)&on, sizeof(on));
+	}
+
 	/*
 	 * We are running from inetd so process stdin.
 	 */
@@ -397,9 +407,9 @@ static int bind_to_port(ipaddr_t bind_ip, short bind_port)
 	 * Set the SO_REUSEADDR option for debugging.
 	 */
 	{
-	 	int	one = 1;
+	 	int	on = 1;
 
-		setsockopt(sock, SOL_SOCKET, SO_REUSEADDR, (char *)&one, sizeof(one));
+		setsockopt(sock, SOL_SOCKET, SO_REUSEADDR, (char *)&on, sizeof(on));
 	}
 #endif
 
@@ -464,6 +474,13 @@ static int connect_to_proxy(ipaddr_t ip, short port)
 		return (-1);
 	}
 
+	/*
+	 * Set the keepalive socket option to on.
+	 */
+	{
+		int		on = 1;
+		setsockopt(STDIN_FILENO, SOL_SOCKET, SO_KEEPALIVE, (char *)&on, sizeof(on));
+	}
 	return (sock);
 }
 
@@ -710,6 +727,11 @@ static void trans_proxy(int sock, struct sockaddr_in *from_addr,
 		FD_SET(proxy, &read_fd);
 		max_fd = FD_MAX(sock, proxy);
 
+		/*
+		 * Allow 5 minutes for any activity.
+		 */
+		alarm(300);
+
 		/*
 		 * Wait for some data to be read.
 		 */
@@ -720,6 +742,11 @@ static void trans_proxy(int sock, struct sockaddr_in *from_addr,
 			return;
 		}
 
+		/*
+		 * We have something so disable the alarm.
+		 */
+		alarm(0);
+
 		/*
 		 * See if any data can be read from the client.
 		 */
@@ -792,6 +819,11 @@ static int	get_site_from_headers(int sock, char *hostname, int hostlen,
 	char	*start;
 	char	*end;
 
+	/*
+	 * Allow 2 minutes to read in the headers.
+	 */
+	alarm(120);
+
 	/*
 	 * Read in a buffers worth of data, hopefully all of the headers
 	 * will be contained in this initial read. This is tagged with a
@@ -852,3 +884,12 @@ static int	get_site_from_headers(int sock, char *hostname, int hostlen,
 	 */
 	return (0);
 }
+
+/*
+ * Catch alarm signals and exit.
+ */
+static void alarm_signal(int sig)
+{
+	syslog(LOG_DEBUG, "Alarm signal caught - connection timeout");
+	exit(1);
+}
